import axios from 'axios';

const API_URL = process.env.REACT_APP_API_URL || 'http://localhost:3000/api';
const WS_URL = process.env.REACT_APP_WS_URL || 'ws://localhost:3000';

// WebSocket connection
let socket = null;
let isConnecting = false;
let reconnectTimer = null;
let eventListeners = {
  'connection': [],
  'new-score': [],
  'delete-score': []
};

// Configuration d'axios
const api = axios.create({
  baseURL: API_URL,
  headers: {
    'Content-Type': 'application/json'
  }
});

// Authentification
export const register = async (username, password) => {
  try {
    const response = await api.post('/register', { username, password });
    return response.data;
  } catch (error) {
    throw error.response ? error.response.data : error;
  }
};

export const login = async (username, password) => {
  try {
    const response = await api.post('/login', { username, password });
    // Stocker les infos utilisateur dans localStorage
    localStorage.setItem('framedUser', JSON.stringify(response.data));
    return response.data;
  } catch (error) {
    throw error.response ? error.response.data : error;
  }
};

export const logout = () => {
  localStorage.removeItem('framedUser');
};

// Gestion des scores
export const addScore = async (userId, date, gameType, gameNumber, score, comment) => {
  try {
    const response = await api.post('/scores', { 
      userId, date, gameType, gameNumber, score, comment 
    });
    return response.data;
  } catch (error) {
    throw error.response ? error.response.data : error;
  }
};

export const getUserScores = async (userId) => {
  try {
    const response = await api.get(`/scores/user/${userId}`);
    return response.data;
  } catch (error) {
    throw error.response ? error.response.data : error;
  }
};

export const deleteScore = async (scoreId, userId) => {
  try {
    const response = await api.delete(`/scores/${scoreId}?userId=${userId}`);
    return response.data;
  } catch (error) {
    throw error.response ? error.response.data : error;
  }
};

// Statistiques et classement
export const getUserStats = async (userId) => {
  try {
    const response = await api.get(`/stats/${userId}`);
    return response.data;
  } catch (error) {
    throw error.response ? error.response.data : error;
  }
};

export const getLeaderboard = async (gameType = 'all') => {
  try {
    const response = await api.get(`/leaderboard?gameType=${gameType}`);
    return response.data;
  } catch (error) {
    throw error.response ? error.response.data : error;
  }
};

export const getAllScores = async (gameType = 'all') => {
  try {
    const response = await api.get(`/scores?gameType=${gameType}`);
    return response.data;
  } catch (error) {
    throw error.response ? error.response.data : error;
  }
};

// WebSocket functions
export const connectToWebSocket = () => {
  if (socket || isConnecting) {
    return; // Already connected or connecting
  }
  
  isConnecting = true;
  
  try {
    socket = new WebSocket(WS_URL);
    
    socket.onopen = () => {
      console.log('WebSocket connection established');
      isConnecting = false;
      
      // Clear any reconnect timer
      if (reconnectTimer) {
        clearTimeout(reconnectTimer);
        reconnectTimer = null;
      }
    };
    
    socket.onmessage = (event) => {
      try {
        const message = JSON.parse(event.data);
        console.log('WebSocket message received:', message);
        
        if (message.type && eventListeners[message.type]) {
          eventListeners[message.type].forEach(callback => callback(message.data));
        }
      } catch (error) {
        console.error('Error parsing WebSocket message:', error);
      }
    };
    
    socket.onerror = (error) => {
      console.error('WebSocket error:', error);
    };
    
    socket.onclose = (event) => {
      console.log('WebSocket connection closed:', event.code, event.reason);
      socket = null;
      isConnecting = false;
      
      // Try to reconnect after a delay
      if (!reconnectTimer) {
        reconnectTimer = setTimeout(() => {
          reconnectTimer = null;
          connectToWebSocket();
        }, 5000);
      }
    };
  } catch (error) {
    console.error('Error creating WebSocket connection:', error);
    isConnecting = false;
    
    // Try to reconnect after a delay
    if (!reconnectTimer) {
      reconnectTimer = setTimeout(() => {
        reconnectTimer = null;
        connectToWebSocket();
      }, 5000);
    }
  }
};

export const disconnectFromWebSocket = () => {
  if (socket) {
    socket.close();
    socket = null;
  }
  
  if (reconnectTimer) {
    clearTimeout(reconnectTimer);
    reconnectTimer = null;
  }
};

export const addWebSocketListener = (event, callback) => {
  if (!eventListeners[event]) {
    eventListeners[event] = [];
  }
  eventListeners[event].push(callback);
};

export const removeWebSocketListener = (event, callback) => {
  if (eventListeners[event]) {
    eventListeners[event] = eventListeners[event].filter(cb => cb !== callback);
  }
};
